@@ -1,5 +1,5 @@
 """
-    gui/nanotron
+    picasso.gui.nanotron
     ~~~~~~~~~~~~~~~~~~~~
 
     Graphical user interface for classification using deep learning
@@ -8,54 +8,95 @@
     :copyright: Copyright (c) 2020 Jungmann Lab, MPI of Biochemistry
 """
 
-import os.path as _ospath
+from __future__ import annotations
+
 import os
 import sys
 import traceback
-import importlib, pkgutil
+import importlib
+import pkgutil
 import datetime
-from time import sleep
-
-import matplotlib.pyplot as plt
-import numpy as np
-
-from matplotlib.backends.backend_qt5agg import FigureCanvas as FigureCanvas
-from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
-
 import threading
 import multiprocessing
+import joblib
+import yaml
 import concurrent.futures
+from time import sleep
 
+import numpy as np
+import matplotlib.pyplot as plt
 from sklearn.neural_network import MLPClassifier
 from sklearn.model_selection import train_test_split
 from sklearn.metrics import confusion_matrix
-import joblib
-import yaml
+
 from PyQt5 import QtCore, QtGui, QtWidgets
 from PyQt5.QtGui import QIcon
 
 from .. import io, lib, render, nanotron, __version__
 
-DEFAULT_MODEL_PATH = _ospath.join(os.sep, "picasso", "model", "default_model.sav")
+DEFAULT_MODEL_PATH = os.path.join(
+    os.sep, "picasso", "model", "default_model.sav",
+)
 default_model = False
 
 
 class Generator(QtCore.QThread):
+    """Class for generating (and optionally augmenting) images from
+    localizations.
+
+    ...
+
+    Attributes
+    ----------
+    classes : list of str
+        The class labels for the localization datasets.
+    expand : bool
+        Whether to expand the dataset by rotating images (augmentation).
+    export : bool
+        Whether to export the generated images.
+    export_paths : list of str
+        Paths to save the exported images.
+    locs_files : dict
+        Each entry is a localization dataset with 'group' field.
+    n_datasets : int
+        Number of localization datasets (classes).
+    oversampling : float
+        Number of display pixels per camera pixel.
+    pick_radius : float
+        Pick radius used for picking.
+
+    Parameters
+    ----------
+    locs : dict
+        Localization datasets with 'group' field.
+    classes : list of str
+        The class labels for the localization datasets.
+    pick_radius : float
+        Pick radius used for picking.
+    oversampling : float
+        Number of display pixels per camera pixel.
+    expand : bool
+        Whether to expand the dataset by rotating images (augmentation).
+    expand_paths : list of str
+        Paths to save the expanded images.
+    parent : QWidget, optional
+        Parent widget for the generator thread. None by default.
+    """
 
     datasets_made = QtCore.pyqtSignal(int, int, int, int)
     datasets_finished = QtCore.pyqtSignal(list, list)
 
     def __init__(
         self,
-        locs,
-        classes,
-        pick_radius,
-        oversampling,
-        expand,
-        export,
-        export_paths,
-        parent=None,
-    ):
+        locs: dict,
+        classes: list[str],
+        pick_radius: float,
+        oversampling: float,
+        expand: bool,
+        export: bool,
+        export_paths: list[str],
+        parent: QtWidgets.QWidget | None = None,
+    ) -> None:
         super().__init__()
         self.locs_files = locs.copy()
         self.pick_radius = pick_radius
@@ -66,20 +107,22 @@ class Generator(QtCore.QThread):
         self.export = export
         self.export_paths = export_paths
 
-    def combine_data_sets(self, X_files, Y_files):
-
+    def combine_data_sets(
+        self,
+        X_files: list[list[np.ndarray]],
+        Y_files: list[list[int]],
+    ) -> tuple[list[np.ndarray], list[int]]:
+        """Merges multiple datasets into one."""
         X = []
         Y = []
         for img in X_files:
             X += img
-
         for label in Y_files:
             Y += label
-
         return X, Y
 
-    def run(self):
-
+    def run(self) -> None:
+        """Generates images from localization datasets."""
         X_files = []
         Y_files = []
 
@@ -91,7 +134,7 @@ class Generator(QtCore.QThread):
             label = self.classes[id]
             n_locs = locs.group.max()
 
-            export_path = _ospath.dirname(self.export_paths[id]) + "/"
+            export_path = os.path.dirname(self.export_paths[id]) + "/"
 
             for c, pick in enumerate(np.unique(locs.group)):
 
@@ -103,7 +146,9 @@ class Generator(QtCore.QThread):
                 )
 
                 if self.export is True and pick < 10:
-                    filename = str(label).replace(" ", "_").lower() + "-" + str(pick)
+                    filename = (
+                        str(label).replace(" ", "_").lower() + "-" + str(pick)
+                    )
                     plt.imsave(
                         export_path + filename + ".png",
                         (10 * pick_img - 1),
@@ -124,10 +169,18 @@ class Generator(QtCore.QThread):
                         data.append(rot_img)
 
                     self.datasets_made.emit(
-                        id + 1, self.n_datasets, splits * (c + 1), splits * (n_locs + 1)
+                        id + 1,
+                        self.n_datasets,
+                        splits * (c + 1),
+                        splits * (n_locs + 1),
                     )
                 else:
-                    self.datasets_made.emit(id + 1, self.n_datasets, c + 1, n_locs + 1)
+                    self.datasets_made.emit(
+                        id + 1,
+                        self.n_datasets,
+                        c + 1,
+                        n_locs + 1,
+                    )
 
                 pick_img = nanotron.prepare_img(
                     pick_img, img_shape=img_shape, alpha=10, bg=1
@@ -140,28 +193,73 @@ class Generator(QtCore.QThread):
             Y_files.append(labels)
 
         X_train, Y_train = self.combine_data_sets(X_files, Y_files)
-
         self.datasets_finished.emit(X_train, Y_train)
 
 
 class Trainer(QtCore.QThread):
+    """Trains a machine learning model.
+
+    ...
+
+    Attributes
+    ----------
+    activation : {'relu', 'identity', 'logistic', 'tanh'}
+        The activation function to use.
+    cm_list : list
+        A list to store the confusion matrices.
+    iterations : int
+        The number of training iterations.
+    learning_rate : float
+        The learning rate for the optimizer.
+    mlp_list : list
+        A list to store the MLPClassifier instances.
+    network : MLPClassifier
+        The neural network model.
+    solver : {'lbfgs', 'sgd', 'adam'}
+        The solver for weight optimization.
+    X_train, X_test : np.ndarray
+        The training and testing data.
+    Y_train, Y_test : np.ndarray
+        The training and testing labels.
+
+    Parameters
+    ----------
+    X_train : np.ndarray
+        The training data.
+    Y_train : np.ndarray
+        The training labels.
+    parameter : dict
+        A dictionary containing the model parameters. See class
+        attributes for details.
+    parent : QWidget, optional
+        The parent widget. None by default.
+    """
 
     training_finished = QtCore.pyqtSignal(list, float, float, list)
 
-    def __init__(self, X_train, Y_train, parameter, parent=None):
+    def __init__(
+        self,
+        X_train: np.ndarray,
+        Y_train: np.ndarray,
+        parameter: dict,
+        parent: QtWidgets.QWidget | None = None,
+    ) -> None:
         super().__init__()
         self.network = parameter["network"]
         self.activation = parameter["activation"]
         self.iterations = parameter["iterations"]
         self.learning_rate = parameter["learning_rate"]
         self.solver = parameter["solver"]
-        self.X_train, self.X_test, self.Y_train, self.Y_test = train_test_split(
+        (
+            self.X_train, self.X_test, self.Y_train, self.Y_test
+        ) = train_test_split(
             X_train, Y_train, test_size=0.30, random_state=42
         )
         self.mlp_list = []  # container to carry mlp class
         self.cm_list = []  # container to carry confusion_matrix
 
-    def run(self):
+    def run(self) -> None:
+        """Train the MLPClassifier model."""
         self.mlp_list = []
         hidden_layer_sizes = tuple(self.network.values())
 
@@ -188,55 +286,94 @@ class Trainer(QtCore.QThread):
         cm = confusion_matrix(self.Y_test, Y_pred)
         self.cm_list.append(cm)
 
-        self.training_finished.emit(self.mlp_list, score, test_score, self.cm_list)
+        self.training_finished.emit(
+            self.mlp_list, score, test_score, self.cm_list,
+        )
 
 
 class Predictor(QtCore.QThread):
+    """Predicts the labels for the input data using the trained model.
+
+    ...
+
+    Attributes
+    ----------
+    locs : np.recarray
+        The input localizations for the model. Must contain the 'group'
+        field.
+    model : MLPClassifier
+        The trained MLPClassifier model.
+    n_groups : int
+        Number of groups, i.e., picks.
+    n_locs : int
+        Number of localizations.
+    oversampling : float
+        Number of display pixels per camera pixel.
+    pick_radius : float
+        The radius used for picking localizations.
+    prediction : np.recarray
+        The predicted labels for the input localizations, one per group,
+        i.e., pick.
+
+    Parameters
+    ----------
+    mlp : MLPClassifier
+        The trained MLPClassifier model.
+    locs : np.recarray
+        The input localizations for the model. Must contain the 'group'
+        field.
+    pick_radius : float
+        The radius used for picking localizations.
+    oversampling : float
+        The factor by which to oversample the localizations.
+    parent : QWidget, optional
+        The parent widget. None by default.
+    """
 
     predictions_made = QtCore.pyqtSignal(int, int)
     prediction_finished = QtCore.pyqtSignal(np.recarray)
 
-    def __init__(self, mlp, locs, pick_radius, oversampling, parent=None):
+    def __init__(
+        self,
+        mlp: MLPClassifier,
+        locs: np.recarray,
+        pick_radius: float,
+        oversampling: float,
+        parent: QtWidgets.QWidget | None = None,
+    ) -> None:
         super().__init__()
         self.model = mlp
         self.locs = locs.copy()
         self.pick_radius = pick_radius
         self.oversampling = oversampling
-
         self.n_locs = len(self.locs["group"])
         self.prediction = np.zeros(
             len(np.unique(self.locs["group"])),
             dtype=[("group", "u4"), ("prediction", "i4"), ("score", "f4")],
         )
-
         self.prediction["group"] = np.unique(self.locs["group"])
-
         self.n_groups = len(np.unique(self.locs["group"]))
 
-        self.p_locs = np.zeros(
-            len(self.locs["group"]),
-            dtype=[("group", "u4"), ("prediction", "i4"), ("score", "f4")],
-        )
-
-    def checkConsecutive(self, l):
-        n = len(l) - 1
-        return sum(np.diff(sorted(l)) == 1) >= n
+    def checkConsecutive(self, ll: np.ndarray) -> bool:
+        n = len(ll) - 1
+        return sum(np.diff(sorted(ll)) == 1) >= n
 
     def _worker(
         self,
-        mlp,
-        locs,
-        picks,
-        pick_radius,
-        oversampling,
-        current,
-        lock,
-        n_picks,
-        predicitions,
-        probabilities,
-        finished,
-    ):
-
+        mlp: MLPClassifier,
+        locs: np.recarray,
+        picks: np.ndarray,
+        pick_radius: float,
+        oversampling: float,
+        current: list[int],
+        lock: threading.Lock,
+        n_picks: int,
+        predictions: np.ndarray,
+        probabilities: np.ndarray,
+        finished: list[int],
+    ) -> None:
+        """Worker thread for making predictions allowing for parallel
+        execution."""
         while True:
             with lock:
                 index = current[0]
@@ -252,14 +389,22 @@ class Predictor(QtCore.QThread):
                 pick_radius=pick_radius,
                 oversampling=oversampling,
             )
-            predicitions[index] = pred[0]
+            predictions[index] = pred[0]
             probabilities[index] = pred_proba.max()
 
             with lock:
                 finished[0] += 1
 
-    def predict_async(self, model, locs, picks, pick_radius, oversampling):
-
+    def _predict_async(
+        self,
+        model: MLPClassifier,
+        locs: np.recarray,
+        picks: np.ndarray,
+        pick_radius: float,
+        oversampling: float,
+    ) -> None:
+        """Make predictions asynchronously, i.e., using
+        multiprocessing."""
         n_picks = len(picks)
 
         predictions = np.zeros(n_picks)
@@ -269,7 +414,7 @@ class Predictor(QtCore.QThread):
 
         n_workers = min(
             60, max(1, int(0.75 * multiprocessing.cpu_count()))
-        ) # Python crashes when using >64 cores
+        )  # Python crashes when using >64 cores
 
         current = [0]
         finished = [0]
@@ -293,12 +438,12 @@ class Predictor(QtCore.QThread):
         executor.shutdown(wait=False)
         return current, predictions, probabilities, finished
 
-    def run(self):
-
+    def run(self) -> None:
+        """Runs predictions."""
         N = len(self.prediction["group"])
         picks = self.prediction["group"]
 
-        current, predictions, probabilities, finished = self.predict_async(
+        current, predictions, probabilities, finished = self._predict_async(
             self.model, self.locs, picks, self.pick_radius, self.oversampling
         )
 
@@ -314,31 +459,16 @@ class Predictor(QtCore.QThread):
         self.locs = lib.append_to_rec(
             self.locs, classes[self.locs["group"]], "prediction"
         )
-        self.locs = lib.append_to_rec(self.locs, probas[self.locs["group"]], "score")
+        self.locs = lib.append_to_rec(
+            self.locs, probas[self.locs["group"]], "score"
+        )
 
         self.prediction_finished.emit(self.locs)
 
 
-class GenericPlotWindow(QtWidgets.QTabWidget):
-    def __init__(self, window_title):
-        super().__init__()
-        self.setWindowTitle(window_title)
-        this_directory = os.path.dirname(os.path.realpath(__file__))
-        icon_path = os.path.join(this_directory, "icons", "nanotron.ico")
-        icon = QtGui.QIcon(icon_path)
-        self.setWindowIcon(icon)
-        self.resize(1000, 500)
-        self.figure = plt.Figure()
-        self.canvas = FigureCanvas(self.figure)
-        vbox = QtWidgets.QVBoxLayout()
-        self.setLayout(vbox)
-        vbox.addWidget(self.canvas)
-
-        self.toolbar = NavigationToolbar(self.canvas, self)
-        vbox.addWidget(self.toolbar)
-
-
 class train_dialog(QtWidgets.QDialog):
+    """Dialog for choosing model training parameters."""
+
     def __init__(self, window):
         super().__init__(window)
         self.window = window
@@ -436,7 +566,9 @@ class train_dialog(QtWidgets.QDialog):
         self.iterations.setRange(0, int(1e4))
         self.iterations.setValue(400)
         train_parameter_grid.addWidget(self.iterations, 0, 1)
-        train_parameter_grid.addWidget(QtWidgets.QLabel("Learning Rate:"), 1, 0)
+        train_parameter_grid.addWidget(
+            QtWidgets.QLabel("Learning Rate:"), 1, 0,
+        )
         self.learing_rate = QtWidgets.QDoubleSpinBox()
         self.learing_rate.resize(200, 50)
         self.learing_rate.setRange(0.00001, 10)
@@ -450,7 +582,9 @@ class train_dialog(QtWidgets.QDialog):
         self.train_btn.setDisabled(True)
         self.train_label = QtWidgets.QLabel("")
         self.train_label.setAlignment(QtCore.Qt.AlignCenter)
-        self.learning_curve_btn = QtWidgets.QPushButton("Show Learning History")
+        self.learning_curve_btn = QtWidgets.QPushButton(
+            "Show Learning History"
+        )
         self.learning_curve_btn.clicked.connect(self.show_learning_stats)
         self.learning_curve_btn.setVisible(False)
 
@@ -530,11 +664,13 @@ class train_dialog(QtWidgets.QDialog):
                 ".sav",
             )
 
-            base, ext = _ospath.splitext(fname)
+            base, ext = os.path.splitext(fname)
             fname = base + ".sav"
 
             self.train_log["Model"] = fname
-            self.train_log["Generated by"] = f"Picasso v{__version__} nanoTRON : Train"
+            self.train_log["Generated by"] = (
+                f"Picasso v{__version__} nanoTRON : Train"
+            )
             import sklearn
 
             self.train_log["Scikit-Learn Version"] = sklearn.__version__
@@ -578,7 +714,7 @@ class train_dialog(QtWidgets.QDialog):
     def show_learning_stats(self):
         if self.mlp is not None:
 
-            canvas = GenericPlotWindow("Learning history")
+            canvas = lib.GenericPlotWindow("Learning history", "nanotron")
             canvas.figure.clear()
 
             ax1, ax2 = canvas.figure.subplots(1, 2)
@@ -588,7 +724,9 @@ class train_dialog(QtWidgets.QDialog):
             ax1.set_xlabel("Iterations")
             ax1.set_ylabel("Loss")
 
-            im = ax2.imshow(self.cm, interpolation="nearest", cmap=plt.cm.Blues)
+            im = ax2.imshow(
+                self.cm, interpolation="nearest", cmap=plt.cm.Blues,
+            )
             ax2.figure.colorbar(im, ax=ax2)
             ax2.set(
                 xticks=np.arange(self.cm.shape[1]),
@@ -635,11 +773,13 @@ class train_dialog(QtWidgets.QDialog):
                 self.train_files_grid.addWidget(c, file, 0)
 
                 self.f_btn = QtWidgets.QPushButton("Load File", self)
-                self.f_btn.clicked.connect(lambda _, fi=file: self.load_train_file(fi))
+                self.f_btn.clicked.connect(
+                    lambda _, fi=file: self.load_train_file(fi)
+                )
                 self.train_files_grid.addWidget(self.f_btn, file, 1)
                 self.f_btns.append(self.f_btn)
 
-                la = QtWidgets.QLabel("Name:".format(file))
+                la = QtWidgets.QLabel("Name:")
                 self.train_files_grid.addWidget(la, file, 2)
 
                 id = QtWidgets.QLineEdit(self)
@@ -685,6 +825,13 @@ class train_dialog(QtWidgets.QDialog):
             if "Pick Diameter" in file:
                 diameter = info[num]["Pick Diameter"]
                 radius = diameter / 2
+            if "Pick Diameter (nm)" in file:
+                diameter = info[num]["Pick Diameter (nm)"]
+                if "Pixel Size (nm)" in file:
+                    px_size = info[num]["Pixel Size (nm)"]
+                    radius = (diameter / px_size) / 2
+                else:
+                    radius = diameter / 2
         return radius
 
     def check_set(self):
@@ -719,7 +866,9 @@ class train_dialog(QtWidgets.QDialog):
 
                 if val >= 1.5:
                     print("Dataset {} will be downsampled.".format(key))
-                    self.training_files[key] = file[file["group"] <= median_length]
+                    self.training_files[key] = (
+                        file[file["group"] <= median_length]
+                    )
 
                     msgBox = QtWidgets.QMessageBox(self)
                     msgBox.setIcon(QtWidgets.QMessageBox.Information)
@@ -737,7 +886,9 @@ class train_dialog(QtWidgets.QDialog):
                     msgBox = QtWidgets.QMessageBox(self)
                     msgBox.setIcon(QtWidgets.QMessageBox.Information)
                     msgBox.setWindowTitle("Info")
-                    msgBox.setText("Class {} is to small. Not enough picks".format(key))
+                    msgBox.setText(
+                        f"Class {key} is to small. Not enough picks"
+                    )
                     msgBox.setInformativeText(
                         "Datasets are inbalanced. "
                         "This can cause training artifacts. "
@@ -758,13 +909,17 @@ class train_dialog(QtWidgets.QDialog):
         elif self.check_set():
             self.generator_running = True
 
-            max_key = max(self.pick_radii, key=lambda x: self.pick_radii.get(x))
+            max_key = max(
+                self.pick_radii, key=lambda x: self.pick_radii.get(x)
+            )
             self.pick_radius = self.pick_radii[max_key]
             self.oversampling = self.oversampling_box.value()
 
             self.train_log["Pick Diameter"] = 2 * self.pick_radius
             self.train_log["Oversampling"] = self.oversampling
-            self.train_log["Expand Trainig Set"] = self.expand_training.isChecked()
+            self.train_log["Expand Training Set"] = (
+                self.expand_training.isChecked()
+            )
 
             self.generate_thread = Generator(
                 locs=self.training_files,
@@ -777,7 +932,9 @@ class train_dialog(QtWidgets.QDialog):
             )
 
             self.generate_thread.datasets_made.connect(self.prepare_progress)
-            self.generate_thread.datasets_finished.connect(self.prepare_finished)
+            self.generate_thread.datasets_finished.connect(
+                self.prepare_finished
+            )
             self.generate_thread.start()
 
         else:
@@ -791,10 +948,12 @@ class train_dialog(QtWidgets.QDialog):
             )
             msgBox.exec_()
 
-    def prepare_progress(self, current_dataset, last_dataset, current_img, last_img):
+    def prepare_progress(
+        self, current_dataset, last_dataset, current_img, last_img,
+    ):
 
-        self.progress_sets_label.setText("{}/{}".format(current_dataset, last_dataset))
-        self.progress_imgs_label.setText("{}/{}".format(current_img, last_img))
+        self.progress_sets_label.setText(f"{current_dataset}/{last_dataset}")
+        self.progress_imgs_label.setText(f"{current_img}/{last_img}")
         self.progress_bar.setMaximum(last_img)
         self.progress_bar.setValue(current_img)
 
@@ -824,6 +983,8 @@ class train_dialog(QtWidgets.QDialog):
 
 
 class View(QtWidgets.QLabel):
+    """Custom QLabel for displaying rendered localizations."""
+
     def __init__(self, window):
         super().__init__()
         self.window = window
@@ -832,7 +993,9 @@ class View(QtWidgets.QLabel):
         self.setAcceptDrops(True)
         self._pixmap = None
         self.locs = None
-        self.rubberband = QtWidgets.QRubberBand(QtWidgets.QRubberBand.Rectangle, self)
+        self.rubberband = QtWidgets.QRubberBand(
+            QtWidgets.QRubberBand.Rectangle, self,
+        )
         self.rubberband.setStyleSheet("selection-background-color: white")
 
     def dragEnterEvent(self, event):
@@ -888,6 +1051,8 @@ class View(QtWidgets.QLabel):
 
 
 class Window(QtWidgets.QMainWindow):
+    """Main window."""
+
     def __init__(self, parent=None):
         super().__init__(parent)
         self.setWindowTitle(f"Picasso v{__version__}: nanoTRON")
@@ -958,7 +1123,9 @@ class Window(QtWidgets.QMainWindow):
 
         accuracy_box = QtWidgets.QGroupBox("Filter export")
         accuracy_grid = QtWidgets.QGridLayout(accuracy_box)
-        self.filter_accuracy_btn = QtWidgets.QCheckBox("Filter Probabilities >=")
+        self.filter_accuracy_btn = QtWidgets.QCheckBox(
+            "Filter Probabilities >="
+        )
         self.export_accuracy = QtWidgets.QDoubleSpinBox()
         self.export_accuracy.resize(50, 50)
         self.export_accuracy.setDecimals(2)
@@ -995,6 +1162,8 @@ class Window(QtWidgets.QMainWindow):
         mainWidget.setLayout(self.grid)
         self.setCentralWidget(mainWidget)
 
+        self.plugin_menu = menu_bar.addMenu("Plugins")  # do not delete
+
     def predict(self):
 
         if (self.predicting is False) and (self.model_loaded is True):
@@ -1038,10 +1207,12 @@ class Window(QtWidgets.QMainWindow):
                 msgBox.exec_()
             else:
 
-                canvas = GenericPlotWindow("Probabilities")
+                canvas = lib.GenericPlotWindow("Probabilities", "nanotron")
                 canvas.figure.clear()
 
-                probabilities_per_pick = np.zeros(len(np.unique(self.locs.group)))
+                probabilities_per_pick = np.zeros(
+                    len(np.unique(self.locs.group))
+                )
                 for c, group_number in enumerate(np.unique(self.locs.group)):
                     pick = self.locs[self.locs.group == group_number]
                     pick_score = np.unique(pick.score)[0]
@@ -1077,9 +1248,8 @@ class Window(QtWidgets.QMainWindow):
             )
         else:
             self.status_bar.showMessage(
-                "Predicting... Please wait. From {} picks - predicted {}".format(
-                    total_picks, pick
-                )
+                f"Predicting... Please wait. From {total_picks} picks - "
+                f"predicted {pick}"
             )
 
     def open(self):
@@ -1117,11 +1287,11 @@ class Window(QtWidgets.QMainWindow):
             self.locs_loaded = True
             self.dist_btn.setDisabled(True)
             self.view.update_image()
-            self.status_bar.showMessage("{} picks loaded.".format(str(groups_max)))
+            self.status_bar.showMessage(f"{groups_max} picks loaded.")
 
         if self.model_loaded is False:
             self.status_bar.showMessage(
-                "{} picks loaded. Load model to predict".format(str(groups_max))
+                f"{groups_max} picks loaded. Load model to predict"
             )
         else:
             self.predict_btn.setDisabled(False)
@@ -1224,7 +1394,8 @@ class Window(QtWidgets.QMainWindow):
         pick_regions = {}
 
         checks = (
-            self.classbox_grid.itemAt(i) for i in range(self.classbox_grid.count())
+            self.classbox_grid.itemAt(i)
+            for i in range(self.classbox_grid.count())
         )
         for btn in checks:
 
@@ -1263,7 +1434,9 @@ class Window(QtWidgets.QMainWindow):
             progress.set_value(count)
             count += 1
 
-            filtered_locs = export_locs[export_locs["prediction"] == prediction]
+            filtered_locs = export_locs[
+                export_locs["prediction"] == prediction
+            ]
             n_groups = np.unique(filtered_locs["group"])
 
             if self.regroup_btn.isChecked():
@@ -1272,10 +1445,12 @@ class Window(QtWidgets.QMainWindow):
                 regroup_dict = dict(zip(n_groups, n_new_groups))
                 regroup_map = [regroup_dict[_] for _ in filtered_locs["group"]]
                 filtered_locs["group"] = regroup_map
-                print("Regrouped datatset {} to {} picks.".format(name, len(n_groups)))
+                print(f"Regrouped datatset {name} to {len(n_groups)} picks.")
 
             nanotron_info = self.nanotron_log.copy()
-            nanotron_info.update({"Generated by": f"Picasso v{__version__} Nanotron"})
+            nanotron_info.update(
+                {"Generated by": f"Picasso v{__version__} Nanotron"}
+            )
             info = self.info + [nanotron_info]
 
             out_filename = "_" + name.replace(" ", "_").lower()
@@ -1288,15 +1463,17 @@ class Window(QtWidgets.QMainWindow):
 
                 for pick in np.unique(filtered_locs["group"]):
                     pick_locs = filtered_locs[filtered_locs["group"] == pick]
-                    pick_centers.append(
-                        [float(np.mean(pick_locs.x)), float(np.mean(pick_locs.y))]
-                    )
+                    pick_centers.append([
+                        float(np.mean(pick_locs.x)),
+                        float(np.mean(pick_locs.y)),
+                    ])
 
                 pick_regions["Centers"] = pick_centers
                 pick_regions["Diameter"] = self.model_info["Pick Diameter"]
                 pick_regions["Shape"] = "Circle"
                 pick_out_path = (
-                    os.path.splitext(self.path)[0] + out_filename + "_picks.yaml"
+                    f"{os.path.splitext(self.path)[0]}_{out_filename}_picks"
+                    + ".yaml"
                 )
                 with open(pick_out_path, "w") as f:
                     yaml.dump(pick_regions, f)
@@ -1331,13 +1508,15 @@ def main():
         p = plugin.Plugin(window)
         if p.name == "nanotron":
             p.execute()
-            
+
     window.show()
 
     def excepthook(type, value, tback):
         lib.cancel_dialogs()
         message = "".join(traceback.format_exception(type, value, tback))
-        errorbox = QtWidgets.QMessageBox.critical(window, "An error occured", message)
+        errorbox = QtWidgets.QMessageBox.critical(
+            window, "An error occured", message,
+        )
         errorbox.exec_()
         sys.__excepthook__(type, value, tback)
 
